@@ -3,18 +3,22 @@ use crate::event::Event;
 use crate::events::event_handler::EventHandler;
 use crate::ipc::context::Context;
 use crate::namespaces::namespace::Namespace;
+use crate::protocol::AsyncStreamProtocolListener;
 use crate::IPCBuilder;
 use std::future::Future;
 use std::pin::Pin;
 
-pub struct NamespaceBuilder {
+pub struct NamespaceBuilder<L: AsyncStreamProtocolListener> {
     name: String,
-    handler: EventHandler,
-    ipc_builder: IPCBuilder,
+    handler: EventHandler<L::Stream>,
+    ipc_builder: IPCBuilder<L>,
 }
 
-impl NamespaceBuilder {
-    pub(crate) fn new(ipc_builder: IPCBuilder, name: String) -> Self {
+impl<L> NamespaceBuilder<L>
+where
+    L: AsyncStreamProtocolListener,
+{
+    pub(crate) fn new(ipc_builder: IPCBuilder<L>, name: String) -> Self {
         Self {
             name,
             handler: EventHandler::new(),
@@ -26,7 +30,7 @@ impl NamespaceBuilder {
     pub fn on<F: 'static>(mut self, event: &str, callback: F) -> Self
     where
         F: for<'a> Fn(
-                &'a Context,
+                &'a Context<L::Stream>,
                 Event,
             ) -> Pin<Box<(dyn Future<Output = Result<()>> + Send + 'a)>>
             + Send
@@ -39,7 +43,7 @@ impl NamespaceBuilder {
 
     /// Builds the namespace
     #[tracing::instrument(skip(self))]
-    pub fn build(self) -> IPCBuilder {
+    pub fn build(self) -> IPCBuilder<L> {
         let namespace = Namespace::new(self.name, self.handler);
         self.ipc_builder.add_namespace(namespace)
     }
